package jp.agentec.sinaburocast.action.user;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;

import javax.mail.MessagingException;

import jp.agentec.sinaburocast.action.AbstractAction;
import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.annotation.NoLogin;
import jp.agentec.sinaburocast.common.exception.ExpectedException;
import jp.agentec.sinaburocast.common.exception.SystemException;
import jp.agentec.sinaburocast.common.util.SinaburoViewUtil;
import jp.agentec.sinaburocast.entity.Member;
import jp.agentec.sinaburocast.entity.PasswordMailSend;
import jp.agentec.sinaburocast.form.user.PasswordMailSendForm;
import jp.agentec.sinaburocast.service.MemberService;
import jp.agentec.sinaburocast.service.PasswordMailSendService;

import org.apache.log4j.Logger;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;

/**
 * ＩＤ、パスワード問い合わせ
 *
 * @author choi-c
 *
 */
@NoLogin
public class PasswordMailSendAction extends AbstractAction {
	private final Logger logger = Logger.getLogger(getClass());

	@ActionForm
	public PasswordMailSendForm passwordMailSendForm;

	public MemberService memberService;
	
	public PasswordMailSendService passwordMailSendService;
	
	
	/**
	 * 初期表示
	 */
	@Execute(validator = false)
	public String index() {
		return "/user/passwordMailSend/passwordMailSend.html";
	}
	
	/**
	 * ＩＤ、パスワード問い合わせ押下
	 *
	 * @return
	 * @throws IOException
	 * @throws SystemException
	 * @throws ExpectedException
	 */
	@Execute(validator = true, input = "/user/passwordMailSend/passwordMailSend.html")
	public String passwordRemain() throws IOException, SystemException, ExpectedException,MessagingException, UnsupportedEncodingException {		
		//Eメール存在チェック
		List<Member> memberList =memberService.findByEmail(passwordMailSendForm.email);
		
		if(memberList.isEmpty()){
			//該当するデータがありません。
			addError(SinaburoViewUtil.getMessage("errors.E007"));
			return "/user/passwordMailSend/";
		}
		
		//メールを送り、データを登録する。
		try {
			int retValue = passwordMailSendService.insertPasswordMailSendLogic(memberList.get(0),passwordMailSendForm.email);
			if(retValue < 1){			
				addError(SinaburoViewUtil.getMessage("errors.S000"));
				return "/user/passwordMailSend/";			
			}
		} catch (MessagingException e) {
			logger.warn("Mail send failed." + e.toString());
			addError(SinaburoViewUtil.getMessage("errors.E055"));
			return "/user/passwordMailSend/";			
		}
		
		//ＩＤ、パスワード問い合わせ完了画面
		return "/user/passwordMailSend/passwordMailSendEnd.html";		        
	}
	
    
	/**
	 * ＩＤ、パスワード問い合わせメールのURL押下
	 * 
	 * @return
	 * @throws IOException
	 * @throws SystemException
	 * @throws ExpectedException
	 */
	@Execute(validator = false)
	public String confirm() throws IOException, SystemException, ExpectedException {
		
		if(StringUtil.isBlank(passwordMailSendForm.qUrlId1)){
			passwordMailSendForm.qUrlId1="";
		}
		
		PasswordMailSend passwordMailSend = passwordMailSendService.findByOneTypeTicket(passwordMailSendForm.qUrlId1,SinaburoConstant.MailType.PASSWORD);
		
		//
		if( passwordMailSend == null){
			request.setAttribute("errMsg", SinaburoViewUtil.getMessage("password.error"));
			return "/error/errorMsg.html";
		}
		
		
		passwordMailSendForm.member = memberService.findById(passwordMailSend.memberId);
		
		//テーブル削除
		passwordMailSendService.delete(passwordMailSend);

		return "/user/passwordMailSend/passwordMailSendConfirm.html";
	}

}